library angular2.src.core.compiler.pipeline.default_steps_dart;

import 'package:angular2/change_detection.dart' show ChangeDetection, Parser;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import './property_binding_parser.dart' show PropertyBindingParser;
import './text_interpolation_parser.dart' show TextInterpolationParser;
import './directive_parser.dart' show DirectiveParser;
import './view_splitter.dart' show ViewSplitter;
import './element_binding_marker.dart' show ElementBindingMarker;
import './proto_view_builder.dart' show ProtoViewBuilder;
import './proto_element_injector_builder.dart' show ProtoElementInjectorBuilder;
import './element_binder_builder.dart' show ElementBinderBuilder;
import './resolve_css.dart' show ResolveCss;
import './shim_shadow_dom.dart' show ShimShadowDom;
import 'package:angular2/src/core/compiler/directive_metadata.dart'
    show DirectiveMetadata;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show ShadowDomStrategy, EmulatedScopedShadowDomStrategy;

createDefaultSteps(ChangeDetection changeDetection, Parser parser,
    DirectiveMetadata compiledComponent, List<DirectiveMetadata> directives,
    ShadowDomStrategy shadowDomStrategy, String templateUrl) {
  var steps = [
    new ViewSplitter(parser),
    new ResolveCss(compiledComponent, shadowDomStrategy, templateUrl),
    new PropertyBindingParser(parser),
    new DirectiveParser(directives),
    new TextInterpolationParser(parser),
    new ElementBindingMarker(),
    new ProtoViewBuilder(changeDetection, shadowDomStrategy),
    new ProtoElementInjectorBuilder(),
    new ElementBinderBuilder(parser)
  ];
  if (shadowDomStrategy is EmulatedScopedShadowDomStrategy) {
    var step = new ShimShadowDom(compiledComponent, shadowDomStrategy);
    ListWrapper.push(steps, step);
  }
  return steps;
}
