library angular2.src.forms.directives_dart;

import 'package:angular2/core.dart'
    show Template, Component, Decorator, NgElement, Ancestor, onChange;
import 'package:angular2/src/facade/dom.dart' show DOM;
import 'package:angular2/src/facade/lang.dart' show isBlank, isPresent, CONST;
import 'package:angular2/src/facade/collection.dart'
    show StringMapWrapper, ListWrapper;
import './model.dart' show ControlGroup, Control;
import './validators.dart' as validators;

@CONST()
class ControlValueAccessor {
  readValue(el) {}
  writeValue(el, value) {}
}
@CONST()
class DefaultControlValueAccessor extends ControlValueAccessor {
  DefaultControlValueAccessor() : super() {}
  readValue(el) {
    return DOM.getValue(el);
  }
  writeValue(el, value) {
    DOM.setValue(el, value);
  }
}
@CONST()
class CheckboxControlValueAccessor extends ControlValueAccessor {
  CheckboxControlValueAccessor() : super() {}
  bool readValue(el) {
    return DOM.getChecked(el);
  }
  writeValue(el, bool value) {
    DOM.setChecked(el, value);
  }
}
var controlValueAccessors = {
  "checkbox": new CheckboxControlValueAccessor(),
  "text": new DefaultControlValueAccessor()
};
ControlValueAccessor controlValueAccessorFor(String controlType) {
  var accessor = StringMapWrapper.get(controlValueAccessors, controlType);
  if (isPresent(accessor)) {
    return accessor;
  } else {
    return StringMapWrapper.get(controlValueAccessors, "text");
  }
}
@Decorator(
    lifecycle: const [onChange],
    selector: '[control]',
    bind: const {'controlName': 'control', 'type': 'type'})
class ControlDirective {
  ControlGroupDirective _groupDecorator;
  NgElement _el;
  String controlName;
  String type;
  ControlValueAccessor valueAccessor;
  Function validator;
  ControlDirective(
      @Ancestor() ControlGroupDirective groupDecorator, NgElement el) {
    this._groupDecorator = groupDecorator;
    this._el = el;
    this.validator = validators.nullValidator;
  }
  onChange(_) {
    this._initialize();
  }
  _initialize() {
    this._groupDecorator.addDirective(this);
    var c = this._control();
    c.validator = validators.compose([c.validator, this.validator]);
    if (isBlank(this.valueAccessor)) {
      this.valueAccessor = controlValueAccessorFor(this.type);
    }
    this._updateDomValue();
    DOM.on(this._el.domElement, "change", (_) {
      return this._updateControlValue();
    });
  }
  _updateDomValue() {
    this.valueAccessor.writeValue(this._el.domElement, this._control().value);
  }
  _updateControlValue() {
    this
        ._control()
        .updateValue(this.valueAccessor.readValue(this._el.domElement));
  }
  _control() {
    return this._groupDecorator.findControl(this.controlName);
  }
}
@Decorator(
    selector: '[control-group]', bind: const {'controlGroup': 'control-group'})
class ControlGroupDirective {
  ControlGroup _controlGroup;
  List<ControlDirective> _directives;
  ControlGroupDirective() : super() {
    this._directives = ListWrapper.create();
  }
  set controlGroup(ControlGroup controlGroup) {
    this._controlGroup = controlGroup;
    ListWrapper.forEach(this._directives, (cd) {
      return cd._updateDomValue();
    });
  }
  addDirective(ControlDirective c) {
    ListWrapper.push(this._directives, c);
  }
  Control findControl(String name) {
    return this._controlGroup.controls[name];
  }
}
var FormDirectives = [ControlGroupDirective, ControlDirective];
