library angular2.test.core.compiler.pipeline.resolve_css_spec_dart;

import 'package:angular2/test_lib.dart'
    show describe, beforeEach, expect, it, iit, ddescribe, el, SpyObject, proxy;
import 'package:angular2/src/core/compiler/pipeline/compile_pipeline.dart'
    show CompilePipeline;
import 'package:angular2/src/core/compiler/pipeline/resolve_css.dart'
    show ResolveCss;
import 'package:angular2/src/core/compiler/pipeline/compile_element.dart'
    show CompileElement;
import 'package:angular2/src/core/compiler/pipeline/compile_step.dart'
    show CompileStep;
import 'package:angular2/src/core/compiler/pipeline/compile_control.dart'
    show CompileControl;
import 'package:angular2/src/core/annotations/annotations.dart' show Component;
import 'package:angular2/src/core/compiler/directive_metadata.dart'
    show DirectiveMetadata;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show ShadowDomStrategy;
import 'package:angular2/src/core/compiler/view.dart' show ProtoView;
import 'package:angular2/src/facade/lang.dart' show IMPLEMENTS, Type, stringify;
import 'package:angular2/src/facade/dom.dart' show DOM;
import 'package:angular2/src/facade/async.dart' show PromiseWrapper;

main() {
  describe('ResolveCss', () {
    createPipeline(ShadowDomStrategy strategy) {
      var annotation = new Component(selector: 'selector');
      var meta = new DirectiveMetadata(SomeComponent, annotation);
      var resolveCss = new ResolveCss(meta, strategy, 'http://base');
      return new CompilePipeline([
        new MockStep((parent, current, control) {
          current.inheritedProtoView = new ProtoView(null, null, null);
        }),
        resolveCss
      ]);
    }
    it('it should set ignoreBindings to true for style elements', () {
      var strategy = new DummyStrategy();
      strategy.spy('transformStyleText').andCallFake((a, b, c) {
        return '.css {}';
      });
      strategy.spy('handleStyleElement');
      var pipeline = createPipeline(strategy);
      var results = pipeline.process(el('<div><style></style></div>'));
      expect(results[0].ignoreBindings).toBe(false);
      expect(results[1].ignoreBindings).toBe(true);
    });
    it('should delegate the handling of style elements to the strategy', () {
      var strategy = new DummyStrategy();
      strategy.spy('transformStyleText').andCallFake((a, b, c) {
        return '.css {}';
      });
      strategy.spy('handleStyleElement');
      var pipeline = createPipeline(strategy);
      var template = el('<div></div>');
      var styleEl = el('<style></style>');
      DOM.appendChild(template, styleEl);
      pipeline.process(template);
      expect(strategy.spy('handleStyleElement')).toHaveBeenCalledWith(styleEl);
    });
    it('should handle css transformed synchronously', () {
      var strategy = new DummyStrategy();
      strategy.spy('transformStyleText').andCallFake((css, url, cmp) {
        return '''${css}, ${url}, ${stringify(cmp)}''';
      });
      strategy.spy('handleStyleElement');
      var pipeline = createPipeline(strategy);
      var template = el('<div></div>');
      var styleEl = el('<style>/*css*/</style>');
      DOM.appendChild(template, styleEl);
      var results = pipeline.process(template);
      expect(styleEl).toHaveText('/*css*/, http://base, SomeComponent');
      expect(results[0].inheritedProtoView.stylePromises.length).toBe(0);
    });
    it('should handle css transformed asynchronously', (done) {
      var completer = PromiseWrapper.completer();
      var strategy = new DummyStrategy();
      var futureCss;
      strategy.spy('transformStyleText').andCallFake((css, url, cmp) {
        futureCss = '''${css}, ${url}, ${stringify(cmp)}''';
        return completer.promise;
      });
      strategy.spy('handleStyleElement');
      var pipeline = createPipeline(strategy);
      var template = el('<div></div>');
      var styleEl = el('<style>/*css*/</style>');
      DOM.appendChild(template, styleEl);
      var results = pipeline.process(template);
      expect(styleEl).toHaveText('');
      expect(results[0].inheritedProtoView.stylePromises[0])
          .toBe(completer.promise);
      completer.resolve(futureCss);
      completer.promise.then((_) {
        expect(styleEl).toHaveText('/*css*/, http://base, SomeComponent');
        done();
      });
    });
  });
}
@proxy
class DummyStrategy extends SpyObject implements ShadowDomStrategy {
  noSuchMethod(m) {
    return super.noSuchMethod(m);
  }
}
class SomeComponent {}
class MockStep extends CompileStep {
  Function processClosure;
  MockStep(process) : super() {
    this.processClosure = process;
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    this.processClosure(parent, current, control);
  }
}
